@@ -1,5 +1,5 @@
 # @abstract Send a camera live stream as part of the SMSCON application.
-# @version $Id: live-camera.sh 181 2012-05-22 23:44:03Z yablacky $
+# @version $Id: live-camera.sh 187 2012-05-31 19:59:30Z yablacky $
 # @usage Runs on N900 phone device and is started and stopped by SMSCON on behalf of COM_CAMERA command.
 # @copyright Lutz Schwarz <schwarz.ware@gmx.de>
 # @license GPL V2
@@ -10,15 +10,9 @@ LIVE_VIEWER_PORT=${3:-5000}
 LIVE_VIEWER_URI=udp://${LIVE_VIEWER_HOST}:${LIVE_VIEWER_PORT}
 
 VIDEO_DIMENSIONS="width=160,height=120"
-VIDEO_OUTPUTFORMAT="dsph263enc keyframe-interval=10 ! video/x-h263 ! rtph263pay pt=96"
 if ifconfig|grep wlan>/dev/null
 then
 	VIDEO_DIMENSIONS="width=320,height=240"
-	__VIDEO_OUTPUTFORMAT="ffmpegcolorspace ! gamma gamma=2 !\
-		videobox top=-24 !\
-		cairotextoverlay "text=$(hostname)" halign=right valign=top ypad=-2 xpad=2 !\
-		cairotimeoverlay !\
-		jpegenc"
 fi
 
 /opt/smscon/camctrl --silent --device "$DEV_VIDEO" -aaa 0&
@@ -28,7 +22,7 @@ gst-launch \
 	v4l2camsrc device="$DEV_VIDEO" !\
 	videorate ! video/x-raw-yuv,framerate=15/1,${VIDEO_DIMENSIONS},format='(fourcc)'UYVY !\
 	videoscale method=nearest-neighbour !\
-	${VIDEO_OUTPUTFORMAT} !\
+	dsph263enc keyframe-interval=10 ! video/x-h263 ! rtph263pay pt=96 !\
 	udpsink host=${LIVE_VIEWER_HOST} port=${LIVE_VIEWER_PORT} sync=false&
 GSTPID=$!
 trap "kill -INT $ALSPID $GSTPID" TERM INT
