@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon_master - remote control master for Nokia N900 phone #
-# $Id: smscon_master.py 221 2012-07-01 21:57:32Z yablacky $
+# $Id: smscon_master.py 251 2012-08-12 21:01:58Z yablacky $
 ##############################################################
 
 import os
@@ -76,6 +76,23 @@ def AuthorizedSIMs_ShowList():
         print 'SIM_%02d = %s %s' % ( n + 1, CodeType, CodeValue )
     return True;
 
+def AuthorizedSIMs_IncludesCurrentSIM():
+    """
+    Check if the currently inserted SIM card is present in SIM database.
+    @return bool True if SIM card is authorized.
+    """
+    ICCID = GetPhoneCode('ICCID')
+    if ICCID == None:
+        LOGGER.info('No SIM card present. Not authorized.')
+        return False
+    IMSI = GetPhoneCode('IMSI')
+    simDB = IMSIConfiguration()
+    ok = simDB.IsAuthorizedSIM(ICCID, IMSI)
+    LOGGER.info('The SIM card is %s.' % IF(ok, 'authorized', 'not authorized'))
+    if IMSI == None:
+        LOGGER.warning('The SIM card is not active (no or wrong PIN).')
+    return ok
+
 def AuthorizedSIMs_GrantCurrentSIM():
     """
     Authorize the currently inserted SIM card to SIM database and
@@ -88,11 +105,10 @@ def AuthorizedSIMs_GrantCurrentSIM():
         return False
     IMSI = GetPhoneCode('IMSI')
     if IMSI == None:
-        LOGGER.error('The SIM card is not active (no or wrong PIN).')
+        LOGGER.error('The SIM card is not active (no or wrong PIN). Not authorized.')
         return False
     simDB = IMSIConfiguration()
-    simDB.updateCode('IMSI', IMSI, None, Silent = True) # Ensure IMSI code of this SIM is removed from database.
-    ok = simDB.updateCode('ICCID', ICCID, '')
+    ok = simDB.GrantAuthorizationToSIM(ICCID, IMSI)
     if ok:
         daemon.tell(daemon.TELL_RELOAD)
     return ok
@@ -107,16 +123,15 @@ def AuthorizedSIMs_RevokeCurrentSIM():
     if ICCID == None:
         LOGGER.error('No SIM card present.')
         return False
-    simDB = IMSIConfiguration()
-    ok1 = simDB.updateCode('ICCID', ICCID, None)
     IMSI = GetPhoneCode('IMSI')
-    if IMSI == None:
-        ok2 = False
+    simDB = IMSIConfiguration()
+    ok = simDB.RevokeAuthorizationFromSIM(ICCID, IMSI)
+    if ok:
+        daemon.tell(daemon.TELL_RELOAD)
+    elif IMSI == None:
         LOGGER.warning('The SIM card is not active (no or wrong PIN); could still be authorized!')
     else:
-        ok2 = simDB.updateCode('IMSI', IMSI, None, Silent = True)
-    if ok1 or ok2:
-        daemon.tell(daemon.TELL_RELOAD)
+        LOGGER.info('The SIM card was not authorized et all.')
     return ok
 
 def AuthorizedSIMs_RevokeAllSIM():
@@ -125,9 +140,8 @@ def AuthorizedSIMs_RevokeAllSIM():
     tell daemon to reload SIM database. 
     @return bool True iff no SIM codes present in SIM database.
     """
-    if not IsFileReadable(InstPath, CodeFile):
-        return True
-    ok = DeleteFile(InstPath, CodeFile)
+    simDB = IMSIConfiguration()
+    ok = simDB.RevokeAuthorizationFromAllSIM()
     if ok:
         daemon.tell(daemon.TELL_RELOAD)
     return ok
@@ -203,7 +217,7 @@ def ShowHiddenOptions():
     print '   -init!             : Create default %s files. Do not upgrade existing ones.' % ' & '.join((ConfigDisplayName, ScriptDisplayName))
     print '   -reload            : Tell daemon to read config file again (-force-reload is also accepted).'
     print '   -get name "default": Get user setting.'
-    print '   -test "test"       : Run developer test (gps1, gps2, ssh, ssh-off, cam, sms, email1, email2, call, script).'
+    print '   -test "test"       : Run developer test (imsi, gps1, gps2, ssh, ssh-off, cam, sms, email1, email2, call, script).'
     print '   -comtest "command" : Run %s command as if received as SMS message (no prefix/suffix required).' % NAME
     print '   -devconfig password: Show uncrypted %s file. MASTERNUMBER is password.' % ConfigDisplayName
 
@@ -237,7 +251,7 @@ def DoCommandLine():
         if len(Mode) > 3 and Mode[0:2] == '--' and Mode[3:1] != '-':
             Mode = Mode[1:]
         # Allow bare word options to control service (called this way by /etc/init.d/rc) 
-        elif Mode in ('start', 'restart', 'stop', 'reload' 'force-reload') and len(args) == 0:
+        elif Mode in ('start', 'restart', 'stop', 'reload' 'force-reload', 'status') and len(args) == 0:
             Mode = '-' + Mode; 
 
         if   Mode == '-help':       ShowOptions(1) 
@@ -370,11 +384,14 @@ def DoCommandLine():
         elif Mode in('-test', '-comtest'):
             if len(args) == 0: return TellError("%s requires argument." % Mode)
             Func = args.pop(0)
-            if not daemon.runs():
-                daemon.start(Mode, Func)
-            elif not daemon.writeCliCommand(Mode, Func):
-                return TellError('%s %s failed to start.' % (Mode[1:], Func))
-            LOGGER.info('%s %s started...' % (Mode[1:], Func))
+            if (Mode == '-test' and Func == 'imsi'):
+                ok = AuthorizedSIMs_IncludesCurrentSIM()
+            else:
+                if not daemon.runs():
+                    daemon.start(Mode, Func)
+                elif not daemon.writeCliCommand(Mode, Func):
+                    return TellError('%s %s failed to start.' % (Mode[1:], Func))
+                LOGGER.info('%s %s started...' % (Mode[1:], Func))
 
         elif Mode == '-alarm':
             if len(args) == 0: return TellError("%s requires argument." % Mode)
