@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon - remote control library for Nokia N900 phone       #
-# $Id: smsconlib.py 221 2012-07-01 21:57:32Z yablacky $
+# $Id: smsconlib.py 251 2012-08-12 21:01:58Z yablacky $
 ##############################################################
 
 # standard modules
@@ -35,7 +35,7 @@ NAME = 'smscon'
 VERSION = '999.1'   # for trunk and private builds etc.
 
 NAME, VERSION = getSysBuildNameAndVersion(NAME, VERSION,
-    headerInfo  = '$Header: https://vcs.maemo.org/svn/smscon/branches/smscon_0.10.6/src/opt/smscon/smsconlib.py 221 2012-07-01 21:57:32Z yablacky $',
+    headerInfo  = '$Header: https://vcs.maemo.org/svn/smscon/branches/smscon_0.10.9/src/opt/smscon/smsconlib.py 251 2012-08-12 21:01:58Z yablacky $',
     dirLevels   = 3 )
     
 XDEBUG   = False        # cross debug (use only if not running on phone)
@@ -436,6 +436,8 @@ class Configuration(dict):
                                     self._upgradableConfigVars[ConfigVar] = True
                                     ErrorMsg = 'Value of %s "%s" needs upgrade.' % (self.ItemTypeName, ConfigVar)
                                 else:
+                                    if type(DefaultValue).__name__ == 'function':
+                                        self._upgradableConfigVars[ConfigVar] = ConfigValue
                                     self._warning('Value of %s "%s" is invalid. ' % (self.ItemTypeName, ConfigVar)
                                                             + self._getPatternMismatchHint(ConfigVar, Pattern))
             if ErrorMsg:
@@ -483,7 +485,7 @@ class Configuration(dict):
                     if not self.Silent:    # don't change ok if silent
                         ok = self._warning('Value of %s "%s" not defined. Using default.' % (self.ItemTypeName, ConfigVar))
                     try:
-                        ConfigValue = temp._calcDefaultValue(DefaultValue)
+                        ConfigValue = temp._calcDefaultValue(DefaultValue, self)
                     except KeyError:
                         if  keyErrorRetry > 0 and keysToBackup:
                             keyErrorRetry -= 1
@@ -542,12 +544,15 @@ class Configuration(dict):
             self._info('Importing %s successful.' % Thing)
         return IF(ok, importCount, -importCount)
 
-    def _calcDefaultValue(self, DefaultValue):  # may throw KeyError exception.
+    def _calcDefaultValue(self, DefaultValue, OldDict = None, NewValue = None):  # may throw KeyError exception.
         val = DefaultValue
         if type(val).__name__ == 'tuple':
             if   len(val)  > 1: val = val[0] % tuple(map(lambda k: self[k], val[1:]))
             elif len(val) == 1: val = self[val[0]]
             else: val = ''
+        elif type(val).__name__ == 'function':
+            OldDict = (type(OldDict).__name__ == 'dict' and OldDict) or self
+            val = val(OldDict, self, IF(NewValue == True, None, NewValue))
         return val
 
     def _genFileValue(self, ConfigValue, VarOptions):
@@ -684,6 +689,7 @@ class Configuration(dict):
         @return int Number of config vars upgraded (added, removed or re-encryped).
         """
         loadCount = 0
+        oldSelf = self.copy()
         keysToUpgrade = sorted(self._ConfigVars.keys())
         keyErrorRetry = len(keysToUpgrade) * 2
         while keysToUpgrade:
@@ -703,7 +709,7 @@ class Configuration(dict):
                         useDefaultValue = True
                 if useDefaultValue:
                     try:
-                        val = self._calcDefaultValue(val)
+                        val = self._calcDefaultValue(val, oldSelf, self._upgradableConfigVars.get(key))
                     except KeyError:
                         if not (keyErrorRetry and keysToUpgrade): raise
                         keyErrorRetry -= 1
@@ -735,7 +741,7 @@ class Configuration(dict):
         if not IsFileReadable(self.Filename):
             self._info('Creating new %s.' % Thing)
         elif Force:
-            self._warning('Forced to create new %s.' % Thing)
+            self._info('Deleting old and creating new %s.' % Thing)
         else:
             self._info('The %s already exists. Not changed.' % Thing)
             return False
@@ -863,10 +869,24 @@ class UserConfiguration(Configuration):
         'SAVEDAUDIBILITY':  ('',                '',                 _OPT_PLAIN, '# Audibility information to be used by COM_SILENTOFF (internal use only):'),
         'SILENCEDEVICE':    (_BoolPattern,      'no',               _OPT_PLAIN, '# Set phone silent before executing command (yes|no):'),
         'AUTODEVICELOCK':   (_BoolPattern,      'no',               _OPT_PLAIN, '# Lock phone before executing any command except un/lock (yes|no):'),
-        'SIMUNLOCK':        (pat('^(no|yes|locknewsim|lockifnopin|lockifnosim|lockalways)$').help('Must be one of "no", "yes", "locknewsim", "lockifnopin", "lockifnosim" or "lockalways"'),
-                                                'no',               _OPT_PLAIN, '# Automatic device un/locking: (no|yes|locknewsim|lockifnopin|lockifnosim|lockalways):'),
-        'STOLENIFNOSIM':    (_BoolPattern,      'yes',              _OPT_PLAIN, '# Activate stolen mode if no SIM is present:'),
         'DEBUGFLAGS':       ('',                '',                 _OPT_PLAIN, '# A csv with debugging options (for development use only):'),
+
+        # Settings for device un/locking on boot
+        'LOCKMODE_OKSIM':   (pat('^(keep|lockifnopin|lockalways)$').help('Must be one of "keep", "lockifnopin" or "lockalways"'),
+                            lambda oldd, curd, newv: IF(oldd.get('SIMUNLOCK')=='lockalways','lockalways','keep'),
+                                                                    _OPT_PLAIN, '# Automatic device locking on authorized SIM: (keep|lockifnopin|lockalways):'),
+        'LOCKMODE_NEWSIM':  (pat('^(keep|unlock|lock)$').help('Must be one of "keep", "unlock", "lock"'),
+                            lambda oldd, curd, newv: {'no':'keep', 'yes':'unlock', 'locknewsim':'lock'}.get(oldd.get('SIMUNLOCK'), 'keep'),
+                                                                    _OPT_PLAIN, '# Automatic device un/locking on unauthorized SIM with wrong PIN: (keep|unlock|lock):'),
+        'LOCKMODE_BADSIM':  (pat('^(keep|unlock|lock)$').help('Must be one of "keep", "unlock", "lock"'),
+                            lambda oldd, curd, newv: IF(oldd.get('SIMUNLOCK')=='lockifnopin','lock','keep'),
+                                                                    _OPT_PLAIN, '# Automatic device un/locking on unauthorized SIM with wrong PIN: (keep|unlock|lock):'),
+        # 'SIMUNLOCK' is obsolete. It is no longer used and defined only for smsconlib's API downward compatibility:
+        'SIMUNLOCK':        (pat('^(no|yes|locknewsim)$').help('Must be one of "no", "yes", "locknewsim"'),
+                            lambda oldd, curd, newv: IF(newv in ('no','yes','locknewsim'), newv, 'no'),
+                                                                    _OPT_PLAIN, '# Automatic device un/locking on unauthorized SIM with right PIN: (no|yes|locknewsim):'),
+        'DEVICELOCKLATE':   (_BoolPattern,      'yes',              _OPT_PLAIN, '# Booting phone should ask for SIM PIN before device-lock code (yes|no):'),
+        'STOLENIFNOSIM':    (_BoolPattern,      'yes',              _OPT_PLAIN, '# Activate stolen mode if no SIM is present:'),
     
         # Sensor detections
         'KEYBOARDDETECT':   (_BoolPattern,      'no',               _OPT_PLAIN, '# Send notification if keyboard is sliding (yes|no):'),
@@ -960,13 +980,18 @@ class IMSIConfiguration(Configuration):
     ConfigPrintFormat   = "%s%s"
     ConfigParseFormat   = "^\\s*(.*?)\\s*()()$"
     TabSize             = 18
+    Loaded              = False
 
     def __init__(self):
         Configuration.__init__(self, Filename = os.path.join(InstPath, CodeFile))
 
     def _genConfigLine(self, ConfigVar, ConfigValue):
         return ConfigVar
-    
+
+    def load(self, **kwargs):
+        self.Loaded = Configuration.load(self, **kwargs)
+        return self.Loaded
+
     def PackCodeType(self, CodeType, ConfigValue):
         """Assemble type and value to a string actually stored in database."""
         if CodeType == 'IMSI':  return ConfigValue
@@ -979,8 +1004,67 @@ class IMSIConfiguration(Configuration):
         else:   return ['IMSI', ConfigValue]
 
     def updateCode(self, CodeType, ConfigName, ConfigValue, Silent = False):
+        self.Loaded = False
         return self.update(self.PackCodeType(CodeType, ConfigName), ConfigValue, Silent)
 
+    def IsAuthorizedSIM(self, ICCID, IMSI):
+        """
+        Check if SIM with given ICCID and IMSI is an authorized SIM.
+        Implicitely convert (upgrade) SIM database: remove IMSI code, store ICCID code.
+        @param string|None ICCID code from SIM.
+        @param string|None IMSI code from SIM.
+        @return bool
+        """
+        if not self.Loaded and not self.load(Silent = True):
+            return False
+        ICCID = ICCID and self.PackCodeType('ICCID', ICCID)
+        IMSI  = IMSI  and self.PackCodeType('IMSI',  IMSI)
+        if ICCID == None:
+            return False    # no IMSI in this case.
+        if ICCID in self.keys():
+            # Ensure only ICCID but no IMSI code stored for this SIM:
+            if IMSI and IMSI in self.keys():
+                self.update(IMSI, None, Silent = True)
+            return True
+        if not (IMSI and IMSI in self.keys()): 
+            return False
+        # Convert entry: remove IMSI code, add ICCID code:
+        self.update(IMSI, None, Silent = True)
+        self.update(ICCID, '')
+        return True
+
+    def GrantAuthorizationToSIM(self, ICCID, IMSI):
+        """
+        Add SIM of specified card codes to SIM database.
+        @param string|None ICCID code from SIM.
+        @param string|None IMSI code from SIM.
+        @return bool True iff SIM card data written to SIM database.
+        """
+        if ICCID == None or IMSI == None:
+            return False
+        self.updateCode('IMSI', IMSI, None, Silent = True) # Ensure IMSI code of this SIM is removed from database.
+        return self.updateCode('ICCID', ICCID, '')
+
+    def RevokeAuthorizationFromSIM(self, ICCID, IMSI):
+        """
+        Remove SIM of specified card codes from SIM database.
+        @param string|None ICCID code from SIM.
+        @param string|None IMSI code from SIM.
+        @return bool True iff specified SIM code no longer present in SIM database.
+        """
+        ok = ICCID != None and self.updateCode('ICCID', ICCID, None)
+        ok = (IMSI != None and self.updateCode('IMSI', IMSI, None, Silent = True)) or ok
+        return ok
+
+    def RevokeAuthorizationFromAllSIM(self):
+        """
+        Remove all SIM card codes from SIM database. 
+        @return bool True iff no SIM codes present in SIM database.
+        """
+        self.Loaded = False
+        return self.create(Force = True)
+
+
 def GetPhoneCode(CodeType):
     """
     Get device hardware IMEI code (always available)
@@ -1385,30 +1469,86 @@ def CreateBootfile():
     BootCode = """\
 description "%(NAME)s - nokia n900 remote control utility"
 author "Lutz Schwarz, Frank Visser"
-version "%(VERSION)s"
+version "$Id: smsconlib.py 251 2012-08-12 21:01:58Z yablacky $"
 
-start on started hildon-desktop
+start on started 00smscon
 stop on starting shutdown
 
+env LATELOCK="/root/.%(NAME)s_latedevlock"
+env BOOTLOCK="/root/.%(NAME)s_bootdevlock"
+
 console none
 
 pre-start script
-    while [ ! -e "%(InstPath)s" ]; do
+    LOG() { test ! -e "%(LogFile)s" || echo $* >>"%(LogFile)s" ; }
+    until [ -e "%(InstPath)s" -a -f /tmp/dbus-info ] ; do
         sleep 3
     done
+    LOG pre-start
+    rm -f "$BOOTLOCK"
+    if [ -f "$LATELOCK" ]
+    then
+        LOG "Doing late dev lock."
+        rm -f "$LATELOCK"
+        nice -2 run-standalone.sh dbus-send --system --type=method_call \\
+            --dest=com.nokia.mce /com/nokia/mce/request \\
+            com.nokia.mce.request.devlock_callback int32:0 || true
+    fi
 end script
 
 script
-    test -e    "%(LogFile)s" || exec "%(DaemonFile)s"
-    >          "%(LogFile)s"
-    chmod 0600 "%(LogFile)s"
+    test -f /tmp/dbus-info && . /tmp/dbus-info   
+    export DBUS_SESSION_BUS_ADDRESS          
+    export DBUS_SESSION_BUS_PID              
+    test -e       "%(LogFile)s" || exec "%(DaemonFile)s"
+    echo start >> "%(LogFile)s"
     exec "%(DaemonFile)s" >> "%(LogFile)s" 2>&1
 end script
 
+post-stop script
+    LOG() { test ! -e "%(LogFile)s" || echo $* >>"%(LogFile)s" ; }
+    LOG post-stop
+    if [ ! -r "%(BootFile)s" ]
+    then
+        rm -f "$LATELOCK"
+        LOG "Late dev lock not possible. Preparing for normal dev lock."    
+    elif [ -r "$BOOTLOCK" ]
+    then
+        LOCKMODE_OKSIM=unknown
+        DEVICELOCKLATE=unknown
+        if [ -r "%(ConfigFile)s" ]
+        then
+            LOCKMODE_OKSIM=$(awk -F \\' "/^\\\\s*LOCKMODE_OKSIM\\\\s*=\\\\s*'.*'/ { print \$2 }" "%(ConfigFile)s")
+            DEVICELOCKLATE=$(awk -F \\' "/^\\\\s*DEVICELOCKLATE\\\\s*=\\\\s*'.*'/ { print \$2 }" "%(ConfigFile)s")
+        fi
+        DEVLOCK="$(cat "$BOOTLOCK")"
+        rm -f "$BOOTLOCK"
+        LOG "Dev lock status = $DEVLOCK, lock mode = $LOCKMODE_OKSIM, lock late = $DEVICELOCKLATE."
+        >"$LATELOCK"
+        if [ -f "$LATELOCK" ] &&
+           [ \( "$DEVICELOCKLATE" == yes -a "$DEVLOCK" != unlocked \) -o "$LOCKMODE_OKSIM" == lockalways ]
+        then
+            run-standalone.sh dbus-send --system --type=method_call \\
+                --dest=com.nokia.mce /com/nokia/mce/request \\
+                com.nokia.mce.request.devlock_callback int32:2 || true
+            test "$LOCKMODE_OKSIM" != lockalways || rm -f "$LATELOCK"
+            LOG "Preparing for late dev lock."
+        else
+            rm -f "$LATELOCK"
+            LOG "Preparing for normal dev lock."
+        fi
+    else
+        rm -f "$LATELOCK"
+        LOG "Dev lock status unknown. Preparing for normal dev lock."
+    fi
+end script
+
 respawn
 """ % {'NAME': NAME, 'VERSION': VERSION, 'InstPath': InstPath,
+       'ConfigFile' : os.path.join(InstPath, ConfigFile),
        'DaemonFile' : os.path.join(InstPath, DaemonFile),
-       'LogFile'    : os.path.join(InstPath, NAME+'_boot.log')
+       'LogFile'    : os.path.join(InstPath, NAME+'_boot.log'),
+       'BootFile'   : os.path.join(BootPath, BootFile)
        }
 
     # Above script does the following: Log the daemon output in smscon_boot.log only
